<?php
/**
 * IDEALIAGroup srl
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@idealiagroup.com so we can send you a copy immediately.
 *
 * @category   MSP
 * @package    MSP_CashOnDelivery
 * @copyright  Copyright (c) 2016 IDEALIAGroup srl (http://www.idealiagroup.com)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

namespace MSP\CashOnDelivery\Model\Total\Creditmemo;

use Magento\Sales\Model\Order\Creditmemo;
use Magento\Tax\Model\Config;
use Magento\Framework\Pricing\PriceCurrencyInterface;

class CashondeliveryTax extends AbstractTotal
{

    /**
     * @var PriceCurrencyInterface
     */
    protected $priceCurrency;

    /**
     * Tax config
     *
     * @var \Magento\Tax\Model\Config
     */
    private $taxConfig;

    public function __construct(
        Config $taxConfig,
        PriceCurrencyInterface $priceCurrency,
        array $data = []
    ) {
        parent::__construct($data);
        $this->taxConfig = $taxConfig;
        $this->priceCurrency = $priceCurrency;
    }


    public function collect(Creditmemo $creditmemo)
    {
        $order = $creditmemo->getOrder();
        $orderTotalQty = $order->getTotalQtyOrdered();
        $creditmemoQty = 0;
        foreach ($creditmemo->getAllItems() as $creditmemoItem) {
            $orderItem = $creditmemoItem->getOrderItem();
            if($orderItem->isDummy() || $creditmemoItem->getQty() <= 0) {
                continue;
            }
            $creditmemoQty += $creditmemoItem->getQty();
        }

        /*
         * Check order quantity if order has bundle with price per row
         */
        $orderItems = $order->getItems();
        $orderHasBundleWithPriceForRow = false;
        $checkOrderTotalQty = 0;
        $bundleItemId = null;
        foreach ($orderItems as $item) {
            if($item->getProductType() == 'bundle')
            {
                $bundleItemId = $item->getItemId();
                if($item->isDummy()){
                    $orderHasBundleWithPriceForRow = true;
                    continue;
                }
            }
            elseif(($item->getProductType() == 'simple' && $item->getParentItemId() == $bundleItemId && $item->isDummy()) || $item->isDummy())
            {
                continue;
            }

            $checkOrderTotalQty+= $item->getQtyOrdered();
        }

        if($orderHasBundleWithPriceForRow) {
            $orderTotalQty = $checkOrderTotalQty;
        }


        // Taxes calculation for gift wrap
        $gwTaxAmount = 0;
        $gwBaseTaxAmount = 0;
        $gwTaxInTotal = false;
        if($creditmemo->hasGwBasePriceInclTax() && ($creditmemo->getGwBasePriceInclTax() == 0 || $creditmemo->getGwBasePriceInclTax() < $order->getGwBasePriceInclTax()))
        {
            if($creditmemo->getGwBasePriceInclTax() < $order->getGwBasePriceInclTax())
            {
                $decreaseFactorGw = $creditmemo->getGwBasePriceInclTax() / $order->getGwBasePriceInclTax();
                $gwTaxAmount = $this->priceCurrency->round($order->getGwTaxAmount() * $decreaseFactorGw);
                $gwBaseTaxAmount = $this->priceCurrency->round($order->getGwBaseTaxAmount() * $decreaseFactorGw, 'base');
                $gwTaxInTotal = true;
            }

            $creditmemo->setGwTaxAmount($gwTaxAmount);
            $creditmemo->setGwBaseTaxAmount($gwBaseTaxAmount);

            if($creditmemo->getGwBasePriceInclTax() == 0)
            {
                $creditmemo->setTaxAmount($creditmemo->getTaxAmount() - $order->getGwTaxAmount());
                $creditmemo->setBaseTaxAmount($creditmemo->getBaseTaxAmount() - $order->getGwBaseTaxAmount());
            }
            else
            {
                $creditmemo->setTaxAmount($creditmemo->getTaxAmount() - $gwTaxAmount);
                $creditmemo->setBaseTaxAmount($creditmemo->getBaseTaxAmount() - $gwBaseTaxAmount);
            }
        }

        if (!$order->getBaseMspCodAmount() || $order->getBaseMspCodAmount() <= 0) {
            $creditmemo->setMspCodTaxAmount(0);
            $creditmemo->setBasePaymentTaxAmount(0);
            return $this;
        }

        $decreaseFactor = $creditmemo->getBaseMspCodAmount() / $order->getBaseMspCodAmount();
        $mspCodTaxAmount = $this->priceCurrency->round($order->getMspCodTaxAmount() * $decreaseFactor);
        $baseMspCodTaxAmount = $this->priceCurrency->round($order->getBaseMspCodTaxAmount() * $decreaseFactor, 'base');
        $creditmemo->setMspCodTaxAmount($mspCodTaxAmount);
        $creditmemo->setBasePaymentTaxAmount($baseMspCodTaxAmount);

        $deltaMspCodTaxAmount = 0;
        $deltaBaseMspCodTaxAmount = 0;
        $mspCodTaxSetted = false;

        /*
         * Fix taxes calculation for MspCod
         * MspCod tax calculation was fixed for entire value of MspCod or if is setted to 0.
         * To check if MspCod value is included in this range.
         */
        if($creditmemo->getBaseMspCodAmount() == 0)
        {
            $creditmemo->setTaxAmount($creditmemo->getTaxAmount() - $mspCodTaxAmount);
            $creditmemo->setBaseTaxAmount($creditmemo->getBaseTaxAmount() - $baseMspCodTaxAmount);
            $mspCodTaxSetted = true;
        }
        else
        {
            /*
             * Partial MspCod refund management
             */
            if(round($mspCodTaxAmount, 3) < round($order->getMspCodTaxAmount(), 3))
            {
                if($orderTotalQty != $creditmemoQty)
                {
                    $creditmemo->setTaxAmount($creditmemo->getTaxAmount() + $mspCodTaxAmount);
                    $mspCodTaxSetted = true;
                }
                else
                {
                    $deltaMspCodTaxAmount = $order->getMspCodTaxAmount() - $mspCodTaxAmount;
                    $creditmemo->setTaxAmount($creditmemo->getTaxAmount() - $deltaMspCodTaxAmount);
                }
            }
            if(round($baseMspCodTaxAmount, 3) < round($order->getBaseMspCodTaxAmount(), 3))
            {
                if($orderTotalQty != $creditmemoQty)
                {
                    $creditmemo->setBaseTaxAmount($creditmemo->getBaseTaxAmount() + $baseMspCodTaxAmount);
                    $mspCodTaxSetted = true;
                }
                else
                {
                    $deltaBaseMspCodTaxAmount = $order->getBaseMspCodTaxAmount() - $baseMspCodTaxAmount;
                    $creditmemo->setBaseTaxAmount($creditmemo->getBaseTaxAmount()  - $deltaBaseMspCodTaxAmount);
                }
            }
        }


        if (!$this->taxConfig->displayCartPricesExclTax($order->getStoreId())) {
            if($orderTotalQty == $creditmemoQty) {
                if($mspCodTaxAmount == 0)
                {
                    $mspCodTaxAmount = $order->getMspCodTaxAmount();
                    $creditmemo->setTaxAmount($creditmemo->getTaxAmount() - $mspCodTaxAmount);
                }
                if($baseMspCodTaxAmount == 0)
                {
                    $baseMspCodTaxAmount = $order->getBaseMspCodTaxAmount();
                    $creditmemo->setBaseTaxAmount($creditmemo->getBaseTaxAmount() - $baseMspCodTaxAmount);
                }
// TEARS 4 FIX  - TODO: required analysis
//
//                $creditmemo->setGrandTotal($creditmemo->getGrandTotal() - $mspCodTaxAmount - $deltaMspCodTaxAmount);
//                $creditmemo->setBaseGrandTotal($creditmemo->getBaseGrandTotal() - $baseMspCodTaxAmount - $deltaBaseMspCodTaxAmount);
//                $creditmemo->setCustomerBalanceReturnMax($creditmemo->getGrandTotal());
//                $creditmemo->setBaseCustomerBalanceReturnMax($creditmemo->getBaseGrandTotal());
            } elseif (!$mspCodTaxSetted) {
                /*
                 * Tax management for partial refund of items and full MspCod
                 */
                $creditmemo->setTaxAmount($creditmemo->getTaxAmount() + $mspCodTaxAmount);
                $creditmemo->setBaseTaxAmount($creditmemo->getBaseTaxAmount() + $baseMspCodTaxAmount);
            }
        }

        return $this;
    }
}
